#!/bin/sh
#   Execute a program for X, shows a message if missing
#   and helps maintain a 'last run' menu in fvwm2

f_runmsg()
{
    l_msg="$1"

    type notify-send > /dev/null 2>&1
    if [ "$?" = "0" ]; then
	notify-send "$l_msg" &
    else
	xmessage -timeout 2 "$l_msg" &
    fi

} # f_runmsg()

#------------------------------------------------------------------------------
# mk_last() -- Regenerate last run file for FVWM, try and generate the file
#------------------------------------------------------------------------------
mk_last()
{

    if test -d "$g_dir_data"
    then
	touch "$g_file_progs" "$g_menu_fmt"
	chmod 644 "$g_file_progs" "$g_menu_fmt"
	grep -v "^${g_prog}" <  "$g_file_progs" \
	    | sed 's/[	 ]*$//g'                \
	    | sed 's/^[	 ]*//'                  \
	    | grep -v '^$' | tail -n 14 > "$g_file_progs".OLD
	echo "$g_full_line" >> "$g_file_progs".OLD
	mv "$g_file_progs".OLD "$g_file_progs"
	#--- reverse order
	if [ "$OS" = "Linux" ]; then
	    tac  "$g_file_progs" > "$g_menu_fmt"
	else
	    tail -r "$g_file_progs" > "$g_menu_fmt"
	fi
	chmod 644 "$g_file_progs" "$g_menu_fmt"
    fi

} # END: mk_last()

###############################################################################
# main
###############################################################################
g_full_line="$@"
g_prog="$1"
g_rm_lock="N"

if [ "$USER" = "" ]; then
    export USER="`id -un`"
fi

export HTHEME=$HOME/.fvwm/cde
g_dir_data=$HTHEME/data
g_menu_fmt=$g_dir_data/last_run_x_fmt.txt
g_file_progs=$g_dir_data/last_run_x.txt
if [ -d "$TMPDIR" ]; then
    xrunlock="$TMPDIR/lock.xrun_prog.lock.$USER"
else
    xrunlock="$g_dir_data/lock.xrun_prog.lock.$USER"
fi

if [ "$OS" = "" ]; then
    export OS=`uname -s`
fi

#------------------------------------------------------------------------------
# validate command
#------------------------------------------------------------------------------
type "$g_prog" > /dev/null 2>&1
err_code="$?"
if [ "$err_code" -ne "0" ]; then
    f_runmsg "Program $g_prog cannot be located"
    exit 0
fi

#------------------------------------------------------------------------------
# log the command and validate lock
# lock file needed to avoid collisions
#------------------------------------------------------------------------------
hour=`date "+%Y%m%d_%H"`

if [ -f "$xrunlock" ]; then
    now=`head -n 1 < "$xrunlock" | cut -c '1-11'`
    #--- if the lock file is old from a different hour, 
    #    process as if it does not exist
    if [ "$hour" = "$now" ]; then
	g_rm_lock="N"
    else
	g_rm_lock="Y"
	date "+%Y%m%d_%H%M%S" > "$xrunlock"
	echo "$g_full_line" >> "$xrunlock"
    fi
else
    g_rm_lock="Y"
    date "+%Y%m%d_%H%M%S" > "$xrunlock"
    echo "$g_full_line" >> "$xrunlock"
fi

if [ "$g_rm_lock" = "Y" ]; then
    mk_last &
fi

#------------------------------------------------------------------------------
# process the command
#------------------------------------------------------------------------------
f_runmsg "STARTING: $g_full_line"

if [ "$g_rm_lock" = "Y" ]; then
    rm -f "$xrunlock"
fi

exec $g_full_line &

exit 0
